://=============================================================================
://
://      phenomenological model of visually evoked spike
://      trains in cat geniculate nonlagged X-cells
://
://=============================================================================
://
://  REMARKS:
://     - phenomenological model makes use of functions and data
://       structures implemented in C (XCellModel.c, calculateXCellResponse.c,
://       nrutil.c, and corresponding .h files);
://       see XCellModel.c for further references
://
://  REFERENCE:
://      [1] N.Gazeres, L.J.Borg-Graham, Y.Fregnac.
://          Visual Neuroscience 15 (1998), 1157-1174.
://      [2] W.H.Press, S.A.Teukolsky, W.T.Vetterling, B.P.Flannery
://          Numerical Recipes in C, Second Edition
://
://=============================================================================
://
://  IMPLEMENTATION: Michael Rudolph, UNIC/CNRS Paris, February 2002
://
://=============================================================================

INDEPENDENT {t FROM 0 TO 1 WITH 1 (ms)}

NEURON {
  SUFFIX XCellResponse
  RANGE Frequency, Spikes, StimulusContrast
}

PARAMETER {
  SimStart        (ms)
  SimStop         (ms)
  SimTimeStep     (ms)
  dt              (ms)
  Vrest           (mV)
  Vspike          (mV)
  SpatialFilter1Sigma
  SpatialFilter2Sigma
  SpatialFilter3Sigma
  SpatialFilter4Sigma
  SpatialFilter5Sigma
  SpatialFilter6Sigma
  SpatialFilter7Sigma
  SpatialFilter8Sigma
  TemporalFilter1A
  TemporalFilter1tau1
  TemporalFilter1tau2
  TemporalFilter2A
  TemporalFilter2tau1
  TemporalFilter2tau2
  TemporalFilter3B
  TemporalFilter3tau
  TemporalFilter4B
  TemporalFilter4tau
  TemporalFilter5A
  TemporalFilter5tau1
  TemporalFilter5tau2
  TemporalFilter6A
  TemporalFilter6tau1
  TemporalFilter6tau2
  TemporalFilter7B
  TemporalFilter7tau
  TemporalFilter8B
  TemporalFilter8tau
  Background
  StimulusType
  StimulusStart
  StimulusEnd
  cspotStimulusD
  barStimulusA
  barStimulusB
  barStimulusX0
  barStimulusY0
  barStimulusThetaDegree
  sgrpThreshold
  sgrpOrder1
  sgrpOrder2
}

ASSIGNED {
  Frequency
  Spikes
  StimulusContrast
}

://=============================================================================

VERBATIM
#include "XCellModel.c"

void initializeSimulationParameters() {
  //-------------------------------------------------------[ Simulation ]-----
  Simulation.tstart = SimStart;
  Simulation.tend = SimStop;
  Simulation.dt = SimTimeStep;
  Simulation.Vrest = Vrest;
  Simulation.Vspike = Vspike;

  //----------------------------------------------------[ SpatialFilter ]-----
  SpatialFilter[1].sigma = SpatialFilter1Sigma;
  SpatialFilter[2].sigma = SpatialFilter2Sigma;
  SpatialFilter[3].sigma = SpatialFilter3Sigma;
  SpatialFilter[4].sigma = SpatialFilter4Sigma;
  SpatialFilter[5].sigma = SpatialFilter5Sigma;
  SpatialFilter[6].sigma = SpatialFilter6Sigma;
  SpatialFilter[7].sigma = SpatialFilter7Sigma;
  SpatialFilter[8].sigma = SpatialFilter8Sigma;

  //---------------------------------------------------[ TemporalFilter ]-----
  TemporalFilter_phasic[1].A    = TemporalFilter1A;
  TemporalFilter_phasic[1].tau1 = TemporalFilter1tau1;
  TemporalFilter_phasic[1].tau2 = TemporalFilter1tau2;
  TemporalFilter_phasic[2].A    = TemporalFilter2A;
  TemporalFilter_phasic[2].tau1 = TemporalFilter2tau1;
  TemporalFilter_phasic[2].tau2 = TemporalFilter2tau2;
  TemporalFilter_tonic[1].B     = TemporalFilter3B;
  TemporalFilter_tonic[1].tau   = TemporalFilter3tau;
  TemporalFilter_tonic[2].B     = TemporalFilter4B;
  TemporalFilter_tonic[2].tau   = TemporalFilter4tau;
  TemporalFilter_phasic[3].A    = TemporalFilter5A;
  TemporalFilter_phasic[3].tau1 = TemporalFilter5tau1;
  TemporalFilter_phasic[3].tau2 = TemporalFilter5tau2;
  TemporalFilter_phasic[4].A    = TemporalFilter6A;
  TemporalFilter_phasic[4].tau1 = TemporalFilter6tau1;
  TemporalFilter_phasic[4].tau2 = TemporalFilter6tau2;
  TemporalFilter_tonic[3].B     = TemporalFilter7B;
  TemporalFilter_tonic[3].tau   = TemporalFilter7tau;
  TemporalFilter_tonic[4].B     = TemporalFilter8B;
  TemporalFilter_tonic[4].tau   = TemporalFilter8tau;

  //----------------------------------------------[ SpontaneousActivity ]-----
  SpontaneousActivity = Background;

  //-------------------------------------------------------[ Simulation ]-----
  Stimulus = StimulusType;
  Stimulus_start = StimulusStart;
  Stimulus_end = StimulusEnd;
  Stimulus_cspot.D = cspotStimulusD;
  Stimulus_bar.a = barStimulusA;
  Stimulus_bar.b = barStimulusB;
  Stimulus_bar.x0 = barStimulusX0;
  Stimulus_bar.y0 = barStimulusY0;
  Stimulus_bar.theta = (double)(barStimulusThetaDegree)*3.141592653589793/(double)180.0;

  //-----------------------------------------------------[ SwitchingGRP ]-----
  SwitchingGRP.Threshold = sgrpThreshold;
  SwitchingGRP.OrderGRP[1] = (int)(sgrpOrder1);
  SwitchingGRP.OrderGRP[2] = (int)(sgrpOrder2);
}
ENDVERBATIM

://=============================================================================

INITIAL {
  VERBATIM
    initializeSimulationParameters();
    initializeXCell();
    calculateXCellResponse();
  ENDVERBATIM
}

BREAKPOINT {
  VERBATIM
    int i;
    i = (int)(t/dt);
    Frequency = FreqOut.out[i];
    Spikes = SpikeOut.out[i];
    if (Stimulus == CSPOT_STIMULUS) StimulusContrast = Stimulus_cspot.I[i];
    else if (Stimulus == BAR_STIMULUS) StimulusContrast = Stimulus_bar.I[i];
  ENDVERBATIM
}

://=============================================================================


