//============================================================================
//
//  function library for phenomenological model of visually
//  evoked spike trains in cat geniculate nonlagged X-cells
//
//============================================================================
//
//  REFERENCE:
//      [1] N.Gazeres, L.J.Borg-Graham, Y.Fregnac.
//          Visual Neuroscience 15 (1998), 1157-1174.
//      [2] W.H.Press, S.A.Teukolsky, W.T.Vetterling, B.P.Flannery
//          Numerical Recipes in C, Second Edition
//
//============================================================================
//
//  IMPLEMENTATION: Michael Rudolph, UNIC/CNRS Paris, February 2002
//
//============================================================================

#include <stdio.h>
#include <math.h>
#include "calculateXCellResponse.h"
#include "nrutil.c"

#define ITMAX   100
#define EPS     3.0e-7
#define FPMIN   1.0e-30
#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS2 1.2e-7
#define RNMX (1.0-EPS2)
#define SMALL 1.0e-3

long idum;

//****************************************************************************
//
//  SPECIAL FUNCTIONS
//
//      reference: [2]
//
//****************************************************************************

//----------------------------------------------------------------------------
// erff                 error function erf(x)
//----------------------------------------------------------------------------

float erff(float x) {
  float gammp(float a, float x);
  return x < 0.0 ? -gammp(0.5,x*x) : gammp(0.5,x*x);
}


//----------------------------------------------------------------------------
// gammln               ln Gamma(xx) for xx>0
//----------------------------------------------------------------------------

float gammln(float xx) {
  double x,y,tmp,ser;
  static double cof[6]={76.18009172947146,-86.50532032941677,
                        24.01409824083091,-1.231739572450155,
                        0.1208650973866179e-2,-0.5395239384953e-5};
  int j;

  y=x=xx;
  tmp=x+5.5;
  tmp -= (x+0.5)*log(tmp);
  ser=1.000000000190015;
  for (j=0;j<=5;j++) ser += cof[j]/++y;
  return -tmp+log(2.5066282746310005*ser/x);
}


//----------------------------------------------------------------------------
// gammp                incomplete gamma function P(a; x)
//----------------------------------------------------------------------------

float gammp(float a, float x) {
  void gcf(float *gammcf, float a, float x, float *gln);
  void gser(float *gamser, float a, float x, float *gln);
  void nrerror(char error_text[]);
  float gamser,gammcf,gln;

  if (x < 0.0 || a <= 0.0) nrerror("Invalid arguments in routine gammp");
  if (x < (a+1.0)) {
    gser(&gamser,a,x,&gln);
    return gamser;
  }
  else {
    gcf(&gammcf,a,x,&gln);
    return 1.0-gammcf;
  }
}


//----------------------------------------------------------------------------
// gcf                  incomplete gamma function Q(a; x) evaluated by its
//                      continued fraction representation as gammcf; also
//                      returns ln Gamma(a) as gln
//----------------------------------------------------------------------------

void gcf(float *gammcf, float a, float x, float *gln) {
  float gammln(float xx);
  void nrerror(char error_text[]);
  int i;
  float an,b,c,d,del,h;

  *gln=gammln(a);
  b=x+1.0-a;
  c=1.0/FPMIN;
  d=1.0/b;
  h=d;
  for (i=1;i<=ITMAX;i++) {
    an = -i*(i-a);
    b += 2.0;
    d=an*d+b;
    if (fabs(d) < FPMIN) d=FPMIN;
    c=b+an/c;
    if (fabs(c) < FPMIN) c=FPMIN;
    d=1.0/d;
    del=d*c;
    h *= del;
    if (fabs(del-1.0) < EPS) break;
  }
  if (i > ITMAX) nrerror("a too large, ITMAX too small in gcf");
  *gammcf=exp(-x+a*log(x)-(*gln))*h;
}


//----------------------------------------------------------------------------
// gser                 incomplete gamma function P(a; x) evaluated by its
//                      series representation as gamser; also returns
//                      ln Gamma(a) as gln
//----------------------------------------------------------------------------

void gser(float *gamser, float a, float x, float *gln) {
  void nrerror(char error_text[]);
  int n;
  float sum,del,ap;

  *gln=gammln(a);
  if (x <= 0.0) {
    if (x < 0.0) nrerror("x less than 0 in routine gser");
    *gamser=0.0;
    return;
  }
  else {
    ap=a;
    del=sum=1.0/a;
    for (n=1;n<=ITMAX;n++) {
      ++ap;
      del *= x/ap;
      sum += del;
      if (fabs(del) < fabs(sum)*EPS) {
        *gamser=sum*exp(-x+a*log(x)-(*gln));
        return;
      }
    }
    nrerror("a too large, ITMAX too small in routine gser");
    return;
  }
}


//----------------------------------------------------------------------------
// convlv               convolves or deconvolves a real data set data[1..n]
//                      with a response function respns[1..n]
//----------------------------------------------------------------------------

void convlv(float data[], unsigned long n, float respns[], unsigned long m, int isign, float ans[]) {
  void realft(float data[], unsigned long n, int isign);
  void twofft(float data1[], float data2[], float fft1[], float fft2[], unsigned long n);
  unsigned long i,no2;
  float dum,mag2,*fft;

  fft=vector(1,n<<1);
  for (i=1;i<=(m-1)/2;i++) respns[n+1-i]=respns[m+1-i];
  for (i=(m+3)/2;i<=n-(m-1)/2;i++) respns[i]=0.0;
  twofft(data,respns,fft,ans,n);
  no2=n>>1;
  for (i=2;i<=n+2;i+=2) {
    if (isign == 1) {
      ans[i-1]=(fft[i-1]*(dum=ans[i-1])-fft[i]*ans[i])/no2;
      ans[i]=(fft[i]*dum+fft[i-1]*ans[i])/no2;
    }
    else if (isign == -1) {
      if ((mag2=SQR(ans[i-1])+SQR(ans[i])) == 0.0) nrerror("Deconvolving at response zero in convlv");
      ans[i-1]=(fft[i-1]*(dum=ans[i-1])+fft[i]*ans[i])/mag2/no2;
      ans[i]=(fft[i]*dum-fft[i-1]*ans[i])/mag2/no2;
    } else nrerror("No meaning for isign in convlv");
  }
  ans[2]=ans[n+1];
  realft(ans,n,-1);
  free_vector(fft,1,n<<1);
}


//----------------------------------------------------------------------------
// realft               Fourier transform of a set of n real-valued
//                      data points
//----------------------------------------------------------------------------

void realft(float data[], unsigned long n, int isign) {
  void four1(float data[],
  unsigned long nn, int isign);
  unsigned long i,i1,i2,i3,i4,np3;
  float c1=0.5,c2,h1r,h1i,h2r,h2i;
  double wr,wi,wpr,wpi,wtemp,theta;
  theta=3.141592653589793/(double) (n>>1);

  if (isign == 1) {
    c2 = -0.5;
    four1(data,n>>1,1);
  }
  else {
    c2=0.5;
    theta = -theta;
  }

  wtemp=sin(0.5*theta);
  wpr = -2.0*wtemp*wtemp;
  wpi=sin(theta);
  wr=1.0+wpr;
  wi=wpi;
  np3=n+3;

  for (i=2;i<=(n>>2);i++) {
    i4=1+(i3=np3-(i2=1+(i1=i+i-1)));
    h1r=c1*(data[i1]+data[i3]);
    h1i=c1*(data[i2]-data[i4]);
    h2r = -c2*(data[i2]+data[i4]);
    h2i=c2*(data[i1]-data[i3]);
    data[i1]=h1r+wr*h2r-wi*h2i;
    data[i2]=h1i+wr*h2i+wi*h2r;
    data[i3]=h1r-wr*h2r+wi*h2i;
    data[i4] = -h1i+wr*h2i+wi*h2r;
    wr=(wtemp=wr)*wpr-wi*wpi+wr;
    wi=wi*wpr+wtemp*wpi+wi;
  }

  if (isign == 1) {
    data[1] = (h1r=data[1])+data[2];
    data[2] = h1r-data[2];
  }
  else {
    data[1]=c1*((h1r=data[1])+data[2]);
    data[2]=c1*(h1r-data[2]);
    four1(data,n>>1,-1);
  }
}


//----------------------------------------------------------------------------
// twofft
//----------------------------------------------------------------------------

void twofft(float data1[], float data2[], float fft1[], float fft2[], unsigned long n) {
  void four1(float data[], unsigned long nn, int isign);
  unsigned long nn3,nn2,jj,j;
  float rep,rem,aip,aim;

  nn3=1+(nn2=2+n+n);

  for (j=1,jj=2;j<=n;j++,jj+=2) {
    fft1[jj-1]=data1[j];
    fft1[jj]=data2[j];
  }

  four1(fft1,n,1);
  fft2[1]=fft1[2];
  fft1[2]=fft2[2]=0.0;

  for (j=3;j<=n+1;j+=2) {
    rep=0.5*(fft1[j]+fft1[nn2-j]);
    rem=0.5*(fft1[j]-fft1[nn2-j]);
    aip=0.5*(fft1[j+1]+fft1[nn3-j]);
    aim=0.5*(fft1[j+1]-fft1[nn3-j]);
    fft1[j]=rep;
    fft1[j+1]=aim;
    fft1[nn2-j]=rep;
    fft1[nn3-j] = -aim;
    fft2[j]=aip;
    fft2[j+1] = -rem;
    fft2[nn2-j]=aip;
    fft2[nn3-j]=rem;
  }
}


//----------------------------------------------------------------------------
// four1                replaces data[1..2*nn] by its discrete
//                      Fourier transform
//----------------------------------------------------------------------------

void four1(float data[], unsigned long nn, int isign) {
  unsigned long n,mmax,m,j,istep,i;
  double wtemp,wr,wpr,wpi,wi,theta;
  float tempr,tempi;

  n=nn << 1;
  j=1;

  for (i=1;i<n;i+=2) {
    if(j > i) {
      SWAP(data[j],data[i]);
      SWAP(data[j+1],data[i+1]);
    }
    m=n >> 1;
    while (m >= 2 && j > m) {
      j -=m;
      m >>= 1;
    }
    j += m;
  }

  mmax=2;
  while (n > mmax) {
    istep=mmax << 1;
    theta=isign*(6.28318530717959/mmax);
    wtemp=sin(0.5*theta);
    wpr = -2.0*wtemp*wtemp;
    wpi=sin(theta);
    wr=1.0;
    wi=0.0;
    for (m=1;m<mmax;m+=2) {
      for (i=m;i<=n;i+=istep) {
        j=i+mmax;
        tempr=wr*data[j]-wi*data[j+1];
        tempi=wr*data[j+1]+wi*data[j];
        data[j]=data[i]-tempr;
        data[j+1]=data[i+1]-tempi;
        data[i] += tempr;
        data[i+1] += tempi;
      }
      wr=(wtemp=wr)*wpr-wi*wpi+wr;
      wi=wi*wpr+wtemp*wpi+wi;
    }
    mmax=istep;
  }
}


//----------------------------------------------------------------------------
// gamdev               returns a deviate distributed as a gamma
//                      distribution of integer order ia
//----------------------------------------------------------------------------

float gamdev(int ia, long *idum) {
  float ran1(long *idum);
  void nrerror(char error_text[]);
  int j;
  float am,e,s,v1,v2,x,y;

  if (ia < 1) nrerror("Error in routine gamdev");
  if(ia < 6) {
    x=1.0;
    for (j=1;j<=ia;j++) x *= ran1(idum);
    x = -log(x);
  }
  else {
    do {
      do {
        do {
          v1=ran1(idum);
          v2=2.0*ran1(idum)-1.0;
        } while (v1*v1+v2*v2 > 1.0);
        y=v2/v1;
        am=ia-1;
        s=sqrt(2.0*am+1.0);
        x=s*y+am;
      } while (x <= 0.0);
      e=(1.0+y*y)*exp(am*log(x/am)-s*y);
    } while (ran1(idum) > e);
  }
  return x;
}


//----------------------------------------------------------------------------
// gamdevTilde          returns a deviate distributed as a
//                      modified gamma distribution (see ref [1], eq.8)
//                      [written by M.R.]
//----------------------------------------------------------------------------

float gamdevTilde(int ia, long *idum) {
  float ran1(long *idum);
  void nrerror(char error_text[]);
  int i, j;
  float sum, am, s, v1, v2, x, y;

  if (ia < 1) nrerror("Error in routine gamdev");

  do {
    do {
      do {
        v1=ran1(idum);
        v2=2.0*ran1(idum)-1.0;
      } while (v1*v1+v2*v2 > 1.0);
      y=v2/v1;
      am=ia-1;
      s=sqrt(2.0*am+1.0);
      x=s*y+am;
    } while (x <= 0.0);
    sum = 0.0;
    for (i=1; i<=ia; i++) {
      s = 1.0;
      for (j=1; j<i; j++) s += log(j);
      sum += (1.0+y*y)*exp(-ia*x + (i-1)*log(ia*x) - s);
    }
  } while (ran1(idum) > sum);
  return x;
}


//----------------------------------------------------------------------------
// ran1                 minimal random number generator of Park and Miller
//                      with Bays-Durham shuffle and added safeguards;
//                      call with idum a negative integer to initialize
//----------------------------------------------------------------------------

float ran1(long *idum) {
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  float temp;

  if (*idum <= 0 || !iy) {
    if (-(*idum) < 1) *idum=1;
    else *idum = -(*idum);
    for (j=NTAB+7;j>=0;j--) {
      k=(*idum)/IQ;
      *idum=IA*(*idum-k*IQ)-IR*k;
      if (*idum < 0) *idum += IM;
      if (j < NTAB) iv[j] = *idum;
    } iy=iv[0];
  } k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if (*idum < 0) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;
}


//****************************************************************************
//
//  SPATIAL FILTER
//
//      reference: [1]
//
//****************************************************************************

//
// spatial filter for bar stimulus
//
void applySpatialFilter_bar(int index, struct sVisualStimulus_bar *stimulus, struct sSpatialFilter *filter, struct sStageOut *output) {
  float s2i, a2, b2, cost, sint, x0p, y0p, fout;

  s2i = 1.0/(sqrt(2.0) * (*filter).sigma);
  a2 = (*stimulus).a/2.0;
  b2 = (*stimulus).b/2.0;
  cost = cos((*stimulus).theta);
  sint = sin((*stimulus).theta);
  x0p = ((*stimulus).x0)*cost + ((*stimulus).y0)*sint;
  y0p = -((*stimulus).x0)*sint + ((*stimulus).y0)*cost;

  fout = 0.25 * (erff((x0p+b2)*s2i) - erff((x0p-b2)*s2i)) * (erff((y0p+a2)*s2i) - erff((y0p-a2)*s2i));
  (*output).out[index] = ((*stimulus).I[index]) * fout ;
  return;
}

//
// spatial filter for cspot stimulus
//
void applySpatialFilter_cspot(int index, struct sVisualStimulus_cspot *stimulus, struct sSpatialFilter *filter, struct sStageOut *output) {
  float a, fout;

  a = ((*stimulus).D)/((*filter).sigma);
  fout = 1 - exp(-a*a/8.0);
  (*output).out[index] = ((*stimulus).I[index]) * fout;
  return;
}


//****************************************************************************
//
//  TEMPORAL FILTER
//
//      reference: [1]
//
//****************************************************************************

//
// temporal filter [phasic]
//
void applyTemporalFilter_phasic(struct sSimulation *sim, struct sStageOut *input, struct sTemporalFilter_phasic *filter, struct sStageOut *output) {
  int i;

  convlv((*input).out, (*sim).maxindex, (*filter).h, ((*sim).maxindex/2)-1, 1, (*output).out);
  for (i=0; i<(*sim).maxindex; i++) (*output).out[i] *= (*sim).dt;
}

//
// temporal filter [tonic]
//
void applyTemporalFilter_tonic(struct sSimulation *sim, struct sStageOut *input, struct sTemporalFilter_tonic *filter, struct sStageOut *output) {
  int i;

  convlv((*input).out, (*sim).maxindex, (*filter).h, ((*sim).maxindex/2)-1, 1, (*output).out);
  for (i=0; i<(*sim).maxindex; i++) (*output).out[i] *= (*sim).dt;
}


//****************************************************************************
//
//  NONLINEAR STAGE
//
//      reference: [1]
//
//****************************************************************************

//
// type 1       _/
//
void applyNonlinearStage_type1(int index, struct sStageOut *input, struct sStageOut *output) {
  float v_NS;
  ((v_NS=(*input).out[index]) < 0) ? ((*output).out[index] = 0) : ((*output).out[index] = v_NS);
}

//
// type 2       ~\
//
void applyNonlinearStage_type2(int index, struct sStageOut *input, struct sStageOut *output) {
  float v_NS;
  ((v_NS=(*input).out[index]) < 0) ? ((*output).out[index] = 0) : ((*output).out[index] = -v_NS);
}

//
// type 3       /~
//
void applyNonlinearStage_type3(int index, struct sStageOut *input, struct sStageOut *output) {
  float v_NS;
  ((v_NS=(*input).out[index]) > 0) ? ((*output).out[index] = 0) : ((*output).out[index] = v_NS);
}

//
// type 4       \_
//
void applyNonlinearStage_type4(int index, struct sStageOut *input, struct sStageOut *output) {
  float v_NS;
  ((v_NS=(*input).out[index]) > 0) ? ((*output).out[index] = 0) : ((*output).out[index] = -v_NS);
}


//****************************************************************************
//
//  SWITCHING GRP
//
//      reference: [1]
//
//****************************************************************************

void applySwitchingGRP(struct sSimulation *sim, struct sSwitchingGRP *sgrp, struct sStageOut *output) {
  double t, tau, tres;
  int sw, order, index;

  t = (*sim).tstart;
  tres = (*sgrp).tres;
  tau = 0.0;
  sw = 1;
  order = 1;
  idum = (*sim).seed;

  do {
    tau += 1000.0*gamdev((*sgrp).OrderGRP[order], &idum)/((*sgrp).OrderGRP[order]);

    if (tau > (*sgrp).Switches[sw]) {
      if (sw < (*sgrp).NumberSwitches) {
        tau = (*sgrp).Switches[sw];
        if (order == 1) order = 2;
        else order = 1;
        sw++;
        tau += 1000.0*gamdevTilde((*sgrp).OrderGRP[order], &idum);
      }
    }

    do {
      t += tres;
      if (t>(*sim).tend) break;
      index = (int)((t+SMALL)/tres)+1;
      (*output).out[index] = (*sim).Vrest;
    } while ((*sgrp).t_tau[index] <= tau);

    (*output).out[index] = (*sim).Vspike;

  } while (t<(*sim).tend);
}
