://=============================================================================
://           
://  minimalExpSyn  -  exponential syanapse (minimal model)
://           
://=============================================================================
://
://  REMARKS:
://    At release time trel, the conductance takes the value gmax 
://    and decays exponentially with time constant tau:
://
://             g(t) = gmax * exp(-(t-trel)/tau)	for t > trel
:// 
://	        g(t) = 0 				for t <= trel
://
://    For a short time interval (Deadtime) after a release, the  
://    synapse can not release again.
:// 
://    Parameters are:
://	     	gmax - maximal conductance
://	    	tau  - synpase time constant
://		deadtime - mimimum time between release events
://		Pthreshold - presyn voltage threshold neccessary for release
://
://-----------------------------------------------------------------------------
://
://  IMPLEMENTATION: Michael Rudolph, UNIC/CNRS Paris, March 2003
://
://=============================================================================

INDEPENDENT {t FROM 0 TO 1 WITH 1 (ms)}

NEURON {
  POINT_PROCESS minimalExpSyn
  POINTER pre
  RANGE g, gmax, lastrelease, tau, deadtime, Erev, Pthreshold
  NONSPECIFIC_CURRENT i
}

UNITS {
  (nA) = (nanoamp)
  (mV) = (millivolt)
  (umho) = (micromho)
}

PARAMETER {
  Pthreshold = 0            	:// voltage level nec for release
  Erev      = 0     (mV)        :// reversal potential
  deadtime  = 0     (ms)        :// mimimum time between release events
  gmax = 0.001      (umho)      :// maximum conductance
  tau = 10	    (ms)	:// synapse time constant
}

ASSIGNED {
  v                 (mV)        :// postsynaptic voltage
  i                 (nA)        :// current = g*(v - Erev)
  g                 (umho)      :// conductance
  pre                           :// pointer to presynaptic variable
  lastrelease       (ms)        :// time of last spike
  R 
  Rmax 
}

INITIAL {
  R = 0
  Rmax = 0
  lastrelease = -1000
}

BREAKPOINT {
  SOLVE release
  g = gmax * R
  i = g*(v - Erev)
}

PROCEDURE release() {
  if (pre > Pthreshold) {    	:// presynaptic threshold reached?
    if ((t - lastrelease) > deadtime) {
      Rmax = R + 1 		:// start new release 
      lastrelease = t
    }
  } 
      
  R = Rmax * exp(-(t-lastrelease)/tau)

  VERBATIM
    return 0;
  ENDVERBATIM
}

://=============================================================================
