://=============================================================================
://  IKm  -  muscarinic potassium current
://=============================================================================
://
://  REFERENCES: Gutfreund Y., Yarom Y., and Segev I.
://              J. Physiol. 483 (1995), 621-640.
://
://              Mainen Z.M., and Sejnowski T.J.
://              Nature 382 (1996), 363-366.
://
://  REMARKS: - Vm-dependent potassium current (muscarinic potassium current)
://           - noninactivating K+ current responsible for the adaptation of
://             firing rate and the afterhyperpolarization (AHP) of cortical
://             pyramidal cells
://           - differential equations (HH-like)
://
://  IMPLEMENTATION: Alain Destexhe, Laval University, 1996
://
://  MODIFICATION: Michael Rudolph, UNIC/CNRS Paris, March 2001
://
://=============================================================================

INDEPENDENT {t FROM 0 TO 1 WITH 1 (ms)}

NEURON {
  SUFFIX IKm
  USEION k READ ek WRITE ik
  RANGE gkbar
  RANGE m_inf, tau_m, m_exp
}

UNITS {
  (mA) = (milliamp)
  (mV) = (millivolt)
}

PARAMETER {
  celsius       (degC)
  ek            (mV)
  gkbar         (mho/cm2)
  dt            (ms)
  v             (mV)
}

STATE {
  m
}

ASSIGNED {
  ik      (mA/cm2)
  m_inf
  tau_m   (ms)
  m_exp
  tadj
}

BREAKPOINT {
  SOLVE states METHOD cnexp
  ik = gkbar * m * (v - ek)
}

://  differential equations for m
://  (use when v changes rapidly)
DERIVATIVE states {
  rates(v)
  m' = (m_inf - m) / tau_m
}

://  exact solution to differential equations for m
://  (use for slowly varying v)
:PROCEDURE states() {
:  rates(v)
:  m = m + m_exp * (m_inf - m)
:}

UNITSOFF

INITIAL {
  ://  original measurements at room temperature
  ://  transformation to actual temperature assuming Q10 = 2.3
  tadj = 2.3 ^ ((celsius-23)/10)
  rates(v)
  m = m_inf
}

PROCEDURE rates( v(mV) ) {  LOCAL a,b
  :// -[ m ]-
  a =  1e-4 * (v+30) / ( 1 - exp(-(v+30)/9) )
  b = -1e-4 * (v+30) / ( 1 - exp( (v+30)/9) )
  tau_m = 1 / (a + b) / tadj
  m_inf = a / (a + b)
  m_exp = 1 - exp(-dt/tau_m)
}

UNITSON

://=============================================================================
