://=============================================================================
://  INa  -  sodium current
://=============================================================================
://
://  REFERENCES: Traub R.D., and Miles R.
://              Neuronal Networks of the Hippocampus.
://              Cambridge, 1991.
://
://  REMARKS: - fast sodium current for action potentials
://           - model of INa for hippocampal CA1 pyramidal cells
://
://  IMPLEMENTATION: Alain Destexhe, Laval University, 1996
://
://  MODIFICATION: Michael Rudolph, UNIC/CNRS Paris, March 2001
://
://=============================================================================

INDEPENDENT {t FROM 0 TO 1 WITH 1 (ms)}

NEURON {
  SUFFIX INa
  USEION na READ ena WRITE ina
  RANGE gnabar, shiftA, shiftI
  RANGE m_inf, tau_m, m_exp
  RANGE h_inf, tau_h, h_exp
}

UNITS {
  (mA) = (milliamp)
  (mV) = (millivolt)
}

PARAMETER {
  gnabar        (mho/cm2)
  shiftA  = 65  (mV)            :// activation shift
  shiftI  = 65  (mV)            :// inactivation shift
  ena           (mV)
  celsius       (degC)
  dt            (ms)
  v             (mV)
}

STATE {
  m h
}

ASSIGNED {
  ina     (mA/cm2)
  m_inf
  h_inf
  tau_m   (ms)
  tau_h   (ms)
  m_exp
  h_exp
  tadj
}

BREAKPOINT {
  SOLVE states METHOD cnexp
  ina = gnabar * m*m*m*h * (v - ena)
}

://  differential equations for m and h
://  (use when v changes rapidly)
DERIVATIVE states {
  rates(v)
  m' = (m_inf - m) / tau_m
  h' = (h_inf - h) / tau_h
}

://  exact solution to differential equations for m and h
://  (use for slowly varying v)
:PROCEDURE states() {
:  rates(v)
:  m = m + m_exp * (m_inf - m)
:  h = h + h_exp * (h_inf - h)
:}

UNITSOFF

INITIAL {
  ://  original measurements at 36 degC
  ://  transformation to actual temperature assuming Q10 = 2.3
  tadj = 2.3 ^ ((celsius-36)/ 10 )
  rates(v)
  m = m_inf
  h = h_inf
}

PROCEDURE rates( v(mV) ) { LOCAL a,b,v2
  :// -[ m ]-
  v2 = v + shiftA
  a = 0.32 * (13-v2) / ( exp((13-v2)/4) - 1)
  b = 0.28 * (v2-40) / ( exp((v2-40)/5) - 1)
  tau_m = 1 / (a + b) / tadj
  m_inf = a / (a + b)
  m_exp = 1 - exp(-dt/tau_m)

  :// -[ h ]-
  v2 = v + shiftI
  a = 0.128 * exp((17-v2)/18)
  b = 4 / ( 1 + exp((40-v2)/5) )
  tau_h = 1 / (a + b) / tadj
  h_inf = a / (a + b)
  h_exp = 1 - exp(-dt/tau_h)
}

UNITSON

://=============================================================================
