://=============================================================================
://  IKd  -  delayed rectifier potassium current
://=============================================================================
://
://  REFERENCES: Traub R.D., and Miles R.
://              Neuronal Networks of the Hippocampus.
://              Cambridge, 1991.
://
://  REMARKS: - Vm-dependent potassium current for action potentials
://           - model of IKd for hippocampal CA1 pyramidal cells
://
://  IMPLEMENTATION: Alain Destexhe, Laval University, 1996
://
://  MODIFICATION: Michael Rudolph, UNIC/CNRS Paris, March 2001
://
://=============================================================================

INDEPENDENT {t FROM 0 TO 1 WITH 1 (ms)}

NEURON {
  SUFFIX IKd
  USEION k READ ek WRITE ik
  RANGE gkbar, shift
  RANGE n_inf, tau_n, n_exp
}

UNITS {
  (mA) = (milliamp)
  (mV) = (millivolt)
}

PARAMETER {
  gkbar         (mho/cm2)
  shift         (mV)
  ek            (mV)
  celsius       (degC)
  dt            (ms)
  v             (mV)
}

STATE {
  n
}

ASSIGNED {
  ik      (mA/cm2)
  n_inf
  tau_n
  n_exp
  tadj
}

BREAKPOINT {
  SOLVE states METHOD cnexp
  ik  = gkbar * n*n*n*n * (v - ek)
}

://  differential equations for n
://  (use when v changes rapidly)
DERIVATIVE states {
  rates(v)
  n' = (n_inf - n) / tau_n
}

://  exact solution to differential equations for n
://  (use for slowly varying v)
:PROCEDURE states() {
:  rates(v)
:  n = n + n_exp * (n_inf - n)
:}

UNITSOFF

INITIAL {
  ://  original measurements at 36 degC
  ://  transformation to actual temperature assuming Q10 = 3
  tadj = 3.0 ^ ((celsius-36)/ 10 )
  rates(v)
  n = n_inf
}

PROCEDURE rates( v(mV) ) { LOCAL a,b,v2
  :// -[ n ]-
  v2 = v + shift
  a = 0.032 * (15-v2) / ( exp((15-v2)/5) - 1)
  b = 0.5 * exp((10-v2)/40)
  tau_n = 1 / (a + b) / tadj
  n_inf = a / (a + b)
  n_exp = 1 - exp(-dt/tau_n)
}

UNITSON

://=============================================================================
